@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -21,7 +21,7 @@ module F = Control_flow_c
 
 let dumb_astcocci_rule_elem = function
  | A.MetaRuleElem _ -> ()
- | A.MetaStmt (ida,_,_,_) -> ()
+ | A.MetaStmt (ida,_,_,_,_) -> ()
  | A.MetaStmtList _ -> ()
  | A.Exp expr -> ()
  | A.TopExp expr -> ()
@@ -52,43 +52,50 @@ let dumb_astcocci_rule_elem = function
  | A.Undef _ -> ()
  | A.Pragma _ -> ()
  | A.Include _ -> ()
+ | A.MetaInclude _ -> ()
  | A.Default _ -> ()
  | A.Case _ -> ()
  | A.AsRe _ -> ()
  | A.DisjRuleElem _ -> failwith "not possible - compiled away in asttoctl"
 
 let dumb_astcocci_decl = function
- | A.UnInit (stg, typa, sa, _)     -> ()
- | A.Init (stg, typa, sa, _, expa, _) -> ()
+   A.UnInit (stg, typa, sa, attr, _)     -> ()
+ | A.Init (stg, typa, sa, attr, _, expa, _) -> ()
  | A.FunProto _ -> ()
  | A.TyDecl (typa, _)     -> ()
  | A.MacroDecl(stg, fn, _, eas, _, _) -> ()
  | A.MacroDeclInit(stg, fn, _, eas, _, _, _, _) -> ()
  | A.MetaDecl _ -> ()
- | A.MetaField _ -> ()
- | A.MetaFieldList _ -> ()
  | A.AsDecl _ -> ()
  | A.Typedef(d,ty1,ty2,pv) -> ()
  | A.DisjDecl xs -> ()
+ | A.ConjDecl xs -> ()
  | A.OptDecl _ -> ()
 
 let dumb_astcocci_initialiser = function (* seems same as the above *)
-    A.Init(stg,ty,id,eq,ini,sem) -> ()
-  | A.UnInit(stg,ty,id,sem) -> ()
+    A.Init(stg,ty,id,attr,eq,ini,sem) -> ()
+  | A.UnInit(stg,ty,id,attr,sem) -> ()
   | A.FunProto _ -> ()
   | A.MacroDecl(_, fn, _, eas, _, _) -> ()
   | A.MacroDeclInit(_, fn, _, eas, _, _, _, _) -> ()
   | A.TyDecl(ty,sem) -> ()
   | A.Typedef(d,ty1,ty2,pv) -> ()
   | A.DisjDecl(decls) -> ()
-  | A.MetaDecl(name,_,_) -> ()
-  | A.MetaField(name,_,_) -> ()
-  | A.MetaFieldList(name,_,_,_) -> ()
+  | A.ConjDecl(decls) -> ()
+  | A.MetaDecl(name,_,_,_) -> ()
   | A.AsDecl(_,_) -> ()
   | A.OptDecl(decl) -> ()
 
+let dumb_astcocci_field = function
+   A.Field (typa, sa, _bf, _)     -> ()
+ | A.MetaField _ -> ()
+ | A.MetaFieldList _ -> ()
+ | A.DisjField xs -> ()
+ | A.ConjField xs -> ()
+ | A.OptField _ -> ()
+
 let dumb_astcocci_expr = function
- | A.MetaExpr (ida,_,_, opttypa, _, _) -> ()
+ | A.MetaExpr (ida,_,_, opttypa, _, _, _bitfield) -> ()
  | A.AsExpr (_,_) -> ()
  | A.AsSExpr (_,_) -> ()
  | A.Edots (_,_) -> ()
@@ -129,10 +136,11 @@ let dumb_astcocci_fulltype = function
     A.Type(_,cv,ty) -> ()
   | A.AsType(_,_) -> ()
   | A.DisjType(types) -> ()
+  | A.ConjType(types) -> ()
   | A.OptType(ty) -> ()
 
 let dumb_astcocci_type = function
- | A.MetaType(ida,_,_) -> ()
+ | A.MetaType(ida,_,_,_) -> ()
  | A.BaseType (basea,strings) -> ()
  | A.SignedT (signa,tya) -> ()
  | A.Pointer (typa, _) -> ()
@@ -143,6 +151,8 @@ let dumb_astcocci_type = function
  | A.EnumDef(ty, lb, ids, rb) -> ()
  | A.StructUnionName(sa, sua) -> ()
  | A.StructUnionDef(ty, lb, decls, rb) -> ()
+ | A.TypeOfExpr(tf, lp, exp, rp) -> ()
+ | A.TypeOfType(tf, lp, ty, rp) -> ()
  | A.TypeName sa -> ()
 
 
