@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -16,6 +16,9 @@ how we reached a particular match *)
 module Ast = Ast_cocci
 
 let extra_counter = ref 0
+
+let reset_fresh_counter () = extra_counter := 0
+
 let get_extra _ =
   let ctr = !extra_counter in
   extra_counter := !extra_counter + 1;
