@@ -190,6 +190,21 @@ module XTRANS = struct
 	match expf expa expb tin with
 	| None -> (* failed *) k expb
 	| Some (x, expb) -> expb);
+      Visitor_c.kdecl_s = (fun (k, bigf) expb ->
+	let iif ii = List.map (Visitor_c.vk_info_s bigf) ii in
+	match expb with
+	  Ast_c.DeclList(xs,iis) ->
+	    (match xs with
+	      (x,ii)::xs ->
+		Ast_c.DeclList
+		  ((Visitor_c.vk_onedecl_s bigf x,iif ii) ::
+		   (List.map
+		      (fun (x,ii) ->
+			(Visitor_c.vk_onedecl_opt_s false bigf x,iif ii))
+		      xs),
+		   iif iis)
+	    | _ -> failwith "no decls")
+	| _ -> k expb);
     }
     in
     Some (expa, Visitor_c.vk_node_s bigf node)
@@ -236,10 +251,10 @@ module XTRANS = struct
 	   Some info ->
 	     failwith
 	       (Printf.sprintf
-		  "weird: dont have position info for the mcodekind in line %d column %d"
+		  "weird: don't have position info for the mcodekind in line %d column %d"
 		  info.Ast_cocci.line info.Ast_cocci.column)
 	 | None ->
-	     failwith "weird: dont have position info for the mcodekind"
+	     failwith "weird: don't have position info for the mcodekind"
 
     (* these remove constraints, at least those that contain pcre regexps,
        which cannot be compared (problem in the unparser) *)
@@ -652,10 +667,6 @@ module XTRANS = struct
       (mk_bigf (maxpos, minpos) (lop,mop,rop,bop))
       x
 
-  let distribute_mck_pragmainfo (maxpos, minpos) = fun (lop,mop,rop,bop) ->
-   fun x ->
-    Visitor_c.vk_pragmainfo_s (mk_bigf (maxpos, minpos) (lop,mop,rop,bop)) x
-
   let distribute_mck_ident_list (maxpos, minpos) = fun (lop,mop,rop,bop) ->
    fun x ->
     Visitor_c.vk_ident_list_splitted_s
@@ -682,7 +693,7 @@ module XTRANS = struct
      | Ast_cocci.CONTEXT (Ast_cocci.DontCarePos,_)
      | Ast_cocci.MINUS   (Ast_cocci.DontCarePos,_,_,_) ->
          Ast_cocci.DontCarePos
-     | _ -> failwith "weird: dont have position info for the mcodekind 2"
+     | _ -> failwith "weird: don't have position info for the mcodekind 2"
 
   let distrf (ii_of_x_f, distribute_mck_x_f) =
     fun ia x -> fun tin ->
@@ -693,7 +704,7 @@ module XTRANS = struct
       (* bug: check_pos mck max && check_pos mck min
        *
        * if do that then if have - f(...); and in C f(1,2); then we
-       * would get a "already tagged" because the '...' would sucess in
+       * would get a "already tagged" because the '...' would success in
        * transformaing both '1' and '1,2'. So being in the range is not
        * enough. We must be equal exactly to the range!
        *)
@@ -735,8 +746,6 @@ module XTRANS = struct
     distrf (Lib_parsing_c.ii_of_cst, distribute_mck_cst)
   let distrf_define_params =
     distrf (Lib_parsing_c.ii_of_define_params,distribute_mck_define_params)
-  let distrf_pragmainfo =
-    distrf (Lib_parsing_c.ii_of_pragmainfo,distribute_mck_pragmainfo)
   let distrf_ident_list =
     distrf (Lib_parsing_c.ii_of_ident_list,distribute_mck_ident_list)
   let distrf_exec_code_list =
@@ -847,7 +856,7 @@ let (transform2: string (* rule name *) -> string list (* dropped_isos *) ->
       let node' = transform_re_node rule_elem node tin in
 
       (* assert that have done something. But with metaruleElem sometimes
-         dont modify fake nodes. So special case before on Fake nodes. *)
+         don't modify fake nodes. So special case before on Fake nodes. *)
       (match F.unwrap node with
       | F.Enter | F.Exit | F.ErrorExit
       | F.EndStatement _ | F.CaseNode _
