@@ -237,7 +237,8 @@ and fullType = typeQualifier * typeC
             and base = CChar2 | CShort | CInt | CLong | CLongLong (* gccext: *)
             and sign = Signed | UnSigned
 
-          and floatType = CFloat | CDouble | CLongDouble
+          and floatType = CFloat | CDouble | CLongDouble |
+	                  CFloatComplex | CDoubleComplex | CLongDoubleComplex
 
 
      (* -------------------------------------- *)
@@ -449,6 +450,7 @@ and statement = statementbis wrap3
   (* cppext: *)
   | MacroStmt
   | Exec of exec_code list
+  | IfdefStmt1 of ifdef_directive list * statement list
 
   and labeled = Label   of name * statement
               | Case    of expression * statement
@@ -632,7 +634,7 @@ and definition = definitionbis wrap (* ( ) { } fakestart sto *)
 and cpp_directive =
   | Define of define
   | Include of includ
-  | Pragma of string wrap * pragmainfo
+  | Pragma of (name * string wrap list) wrap
   | OtherDirective of il
 (*| Ifdef ? no, ifdefs are handled differently, cf ifdef_directive below *)
 
@@ -689,10 +691,6 @@ and include_rel_pos = {
    last_of :  string list list;
  }
 
-and pragmainfo =
-    PragmaTuple of argument wrap2 (* , *) list wrap
-  | PragmaIdList of name wrap2 list (* no commas, wrap2 is always empty *)
-
 (* todo? to specialize if someone need more info *)
 and ifdef_directive = (* or and 'a ifdefed = 'a list wrap *)
   | IfdefDirective of (ifdefkind * matching_tag) wrap
@@ -831,6 +829,9 @@ and metavars_binding = (Ast_cocci.meta_name, metavar_binding_kind) assoc
   | MetaPosValList   of (* current elem, min, max *)
       (Common.filename * string (*element*) * (posl * posl) option *
 	 posl * posl) list
+  | MetaComValList   of (Token_c.comment_like_token list *
+			   Token_c.comment_like_token list *
+			   Token_c.comment_like_token list) list
   | MetaListlenVal   of int
   | MetaNoVal
 
@@ -1002,6 +1003,23 @@ let rewrap_str s ii =
     | FakeTok (_,vpi) -> FakeTok (s,vpi)
     | AbstractLineTok pi -> OriginTok { pi with Common.str = s;})}
 
+let rewrap_charpos charpos ii =
+  {ii with pinfo =
+    (match ii.pinfo with
+      OriginTok pi -> OriginTok { pi with Common.charpos = charpos;}
+    | ExpandedTok (pi,vpi) ->
+	ExpandedTok ({ pi with Common.charpos = charpos;},vpi)
+    | FakeTok (s,vpi) -> FakeTok (s,vpi)
+    | AbstractLineTok pi -> OriginTok { pi with Common.charpos = charpos;})}
+
+let rewrap_col col ii =
+  {ii with pinfo =
+    (match ii.pinfo with
+      OriginTok pi -> OriginTok { pi with Common.column = col;}
+    | ExpandedTok (pi,vpi) -> ExpandedTok ({ pi with Common.column = col;},vpi)
+    | FakeTok (s,vpi) -> FakeTok (s,vpi)
+    | AbstractLineTok pi -> OriginTok { pi with Common.column = col;})}
+
 let rewrap_pinfo pi ii =
   {ii with pinfo = pi}
 
@@ -1081,10 +1099,14 @@ let compare_pos ii1 ii2 =
   match (pos1,pos2) with
     (Real p1, Real p2) ->
       compare p1.Common.charpos p2.Common.charpos
-  | (Virt (p1,_), Real p2) ->
-      if (compare p1.Common.charpos p2.Common.charpos) = (-1) then (-1) else 1
-  | (Real p1, Virt (p2,_)) ->
-      if (compare p1.Common.charpos p2.Common.charpos) = 1 then 1 else (-1)
+  | (Virt (p1,offset), Real p2) ->
+      if p1.Common.charpos = p2.Common.charpos
+      then if offset < 0 then -1 else 1
+      else compare p1.Common.charpos p2.Common.charpos
+  | (Real p1, Virt (p2,offset)) ->
+      if p1.Common.charpos = p2.Common.charpos
+      then if offset < 0 then 1 else -1
+      else compare p1.Common.charpos p2.Common.charpos
   | (Virt (p1,o1), Virt (p2,o2)) ->
       let poi1 = p1.Common.charpos in
       let poi2 = p2.Common.charpos in
@@ -1172,7 +1194,22 @@ let al_comments keep_comments x =
     if keep_comments
     then l
     else
-      List.filter (function (Token_c.TCommentCpp _,_) -> true | _ -> false) l in
+      (* This is a hack.  If we keep something, we need to keep the newlines
+	 involved.  But keeping this
+	 information may mean that metavariables don't match what they are
+	 supposed to, if this information is taken into account.  Actually,
+	 we only want this information for + code.  If we just put l here,
+	 then lots of tests fail, so it seems that metavariables are being
+	 matched again, and not just used for + *)
+      let rec loop = function
+	  [] -> []
+	| ((Token_c.TCommentNewline,_) as x)::
+	  (((Token_c.TCommentCpp _,_)::_) as xs) -> x :: loop xs
+	| ((Token_c.TCommentCpp _,_) as x)::
+	  ((Token_c.TCommentNewline,_) as y)::xs -> x :: y :: loop xs
+	| ((Token_c.TCommentCpp _,_) as x)::xs -> x :: loop xs
+	| x::xs -> loop xs in
+      loop l in
   let al_com (x,i) =
     (x,{i with Common.charpos = magic_real_number;
 	 Common.line = magic_real_number;
@@ -1258,7 +1295,7 @@ let (split_nocomma: 'a list -> ('a, il) either list) = function l ->
 let (unsplit_nocomma: ('a, il) either list -> 'a list) = function l ->
   l +>
   List.map
-    (function Left x -> x | Right x -> failwith "not possible")
+    (function Left x -> x | Right x -> failwith "unsplit: not possible")
 
 (*****************************************************************************)
 (* Helpers, could also be put in lib_parsing_c.ml instead *)
