@@ -251,8 +251,8 @@ let insert_all_braces xs starti nodety str =
  * ce label. De meme, meme le cas simple ou la derniere instruction
  * c'est un return, alors ca va generer un DeadCode :(
  *
- * So make a first pass where dont launch exn at all. Create nodes,
- * if starti is None then dont add arc. Then make a second pass that
+ * So make a first pass where don't launch exn at all. Create nodes,
+ * if starti is None then don't add arc. Then make a second pass that
  * just checks that all nodes (except enter) have predecessors.
  * So make starti an option too. So type is now
  *
@@ -549,7 +549,7 @@ let rec aux_statement : (nodei option * xinfo) -> statement -> nodei option =
               *   cf tests-bis/switch_no_body.c
               * but I don't think it's worthwhile to handle
               * such pathological and rare case. Not worth
-              * the complexity. Safe to assume a coumpound.
+              * the complexity. Safe to assume a compound.
               *)
              raise (Error (WeirdSwitch (pinfo_of_ii [i1])))
        in
@@ -962,6 +962,9 @@ let rec aux_statement : (nodei option * xinfo) -> statement -> nodei option =
       !g +> add_arc_opt (starti, newi);
       Some newi
 
+  | Ast_c.IfdefStmt1 (ifdefs, xs) ->
+      aux_statement_list starti (xi,xi_lbl)
+	[Ast_c.IfdefStmt2 (ifdefs, List.map (fun x -> [StmtElem x]) xs)]
 
   (* ------------------------- *)
   | Ast_c.NestedFunc def ->
@@ -1469,8 +1472,11 @@ let rec ast_to_control_flow e =
       | Ast_c.DefineEmpty ->
           let endi = !g +> add_node EndNode lbl_0 "[end]" in
           !g#add_arc ((headeri, endi),Direct);
-      | Ast_c.DefineInit _ ->
-          raise (Error(Define(pinfo_of_ii ii)))
+      | Ast_c.DefineInit i ->
+          let ei   = !g +> add_node (DefineInit i) lbl_0 "definit" in
+          let endi = !g +> add_node EndNode        lbl_0 "[end]" in
+          !g#add_arc ((headeri, ei) ,Direct);
+          !g#add_arc ((ei, endi) ,Direct);
       | Ast_c.DefineMulti sts -> (* christia: todo *)
           raise (Error(Define(pinfo_of_ii ii)))
       | Ast_c.DefineTodo ->
@@ -1491,9 +1497,9 @@ let rec ast_to_control_flow e =
 
       Some !g
 
-  | Ast_c.CppTop (Ast_c.Pragma ((id,ii), pragmainfo))  ->
-      let elem = PragmaHeader ((id,ii), pragmainfo) in
-      let str = "#pragma " ^ id in
+  | Ast_c.CppTop (Ast_c.Pragma ((id,rest),ii))  ->
+      let elem = PragmaHeader ((id,rest),ii) in
+      let str = "#pragma" in
       let ei =   !g +> add_node elem    lbl_0 str in
       let endi = !g +> add_node EndNode lbl_0 "[end]" in
 
@@ -1643,7 +1649,7 @@ let report_error error =
     match error with
     | DeadCode          infoopt ->
 	(match infoopt with
-	| None -> pr2 "FLOW: deadcode detected, but cant trace back the place"
+	| None -> pr2 "FLOW: deadcode detected, but can't trace back the place"
 	| Some info ->
 	    pr2 ("FLOW: deadcode detected: " ^ error_from_info info))
     | CaseNoSwitch      info ->
@@ -1656,7 +1662,7 @@ let report_error error =
     | NoEnclosingLoop   (info) ->
 	pr2 ("FLOW: can't find enclosing loop: " ^ error_from_info info)
     | GotoCantFindLabel (s, info) ->
-	pr2 ("FLOW: cant jump to " ^ s ^ ": because we can't find this label")
+	pr2 ("FLOW: can't jump to " ^ s ^ ": because we can't find this label")
     | NoExit info ->
 	pr2 ("FLOW: can't find exit or error exit: " ^ error_from_info info)
     | DuplicatedLabel s ->
