@@ -63,6 +63,7 @@ type define_def = string * define_param * define_body
      | HintMacroString
      | HintMacroStatement
      | HintAttribute
+     | HintEndAttribute
      | HintMacroIdentBuilder
 
 
@@ -77,7 +78,8 @@ let assoc_hint_string = [
   "YACFE_STRING"     , HintMacroString;
   "YACFE_STATEMENT"  , HintMacroStatement;
   "YACFE_ATTRIBUTE"  , HintAttribute;
-  "YACFE_IDENT_BUILDER"  , HintMacroIdentBuilder;
+  "YACFE_END_ATTRIBUTE" , HintEndAttribute;
+  "YACFE_IDENT_BUILDER" , HintMacroIdentBuilder;
 
   "MACROSTATEMENT"   , HintMacroStatement; (* backward compatibility *)
 ]
@@ -108,6 +110,7 @@ let (token_from_parsinghack_hint:
        Parser_c.TMacroStmt (s, ii)
    | HintAttribute ->
        Parser_c.TMacroAttr (s, ii)
+   | HintEndAttribute -> failwith "not supported"
    | HintMacroIdentBuilder ->
        Parser_c.TMacroIdentBuilder (s, ii)
 
@@ -189,13 +192,13 @@ let rec remap_keyword_tokens xs =
 let rec get_ident_in_concat_op xs =
   match xs with
   | [] ->
-      pr2 "weird: ident after ## operator not found";
+      pr2_once "weird: ident after ## operator not found";
       "", []
   | [x] ->
       (match x with
       | Parser_c.TIdent (s, i1) -> s, []
       | _ ->
-          pr2 "weird: ident after ## operator not found";
+          pr2_once "weird: ident after ## operator not found";
           "", [x]
       )
   | x::y::xs ->
@@ -206,7 +209,7 @@ let rec get_ident_in_concat_op xs =
       | Parser_c.TIdent (s, i1), _ ->
           s, (y::xs)
       | _ ->
-          pr2 "weird: ident after ## operator not found";
+          pr2_once "weird: ident after ## operator not found";
           "", x::y::xs
       )
 
@@ -231,7 +234,7 @@ let rec agglomerate_concat_op_ident xs =
           let i1' = Ast_c.rewrap_str new_s i1 in
           Parser_c.TIdent (new_s, i1')::agglomerate_concat_op_ident rest_toks
       | Parser_c.TCppConcatOp _, _ ->
-          pr2 "weird, ## alone";
+          pr2_once "weird, ## alone";
           x::agglomerate_concat_op_ident (y::xs)
       | _ ->
           x::agglomerate_concat_op_ident (y::xs)
@@ -258,7 +261,7 @@ let rec agglomerate_concat_op_ident xs =
  * for instance transform some of the back into some TypedefIdent
  * so cpp_engine may be fooled?
  *)
-let rec (cpp_engine:
+let (cpp_engine:
           ?evaluate_concatop:bool ->
           (string , Parser_c.token list) assoc ->
           Parser_c.token list -> Parser_c.token list) =
@@ -301,14 +304,38 @@ let rec (cpp_engine:
  * after fix_token_define a TDefineIdent, no more a TIdent.
  *)
 
-let rec apply_macro_defs
+let around parens (err_line,err_col) =
+  match parens with (* can contain only one element, if open with no close *)
+    lp::_ -> (* parens also contains commas *)
+      let rp = List.hd (List.rev parens) in
+      let lp_line = lp.Token_views_c.line in
+      let lp_col = lp.Token_views_c.col in
+      let rp_line = rp.Token_views_c.line in
+      let rp_col = rp.Token_views_c.col in
+      if (lp_line = rp_line && lp_col <= rp_col) || lp_line < rp_line
+      then
+	(lp_line < err_line || (lp_line = err_line && lp_col < err_col)) &&
+	(rp_line > err_line || (rp_line = err_line && rp_col > err_col))
+      else failwith "unexpected paren order"
+  | _ -> false
+
+let at tok (err_line,err_col) =
+  let tok_line = tok.Token_views_c.line in
+  let tok_col = tok.Token_views_c.col in
+  tok_line = err_line && tok_col = err_col
+
+(* This applies the macro in the current definition and all subsequent ones.
+This is unfortunate when the macro itself introduces a parse error, but it
+seems that we don't know where the current function ends. *)
+
+let apply_macro_defs
  ~msg_apply_known_macro
  ~msg_apply_known_macro_hint
  ?evaluate_concatop
  ?(inplace_when_single=true)
- defs xs =
+ defs pos xs =
 
- let rec apply_macro_defs xs =
+ let rec apply_macro_defs dynamic_macs pos xs =
   match xs with
   | [] -> ()
 
@@ -439,7 +466,55 @@ let rec apply_macro_defs
                 id.tok <- token_from_parsinghack_hint (s,i1) hint;
             )
       );
-      apply_macro_defs xs
+      apply_macro_defs dynamic_macs pos xs
+
+  | PToken {tok = TIdent (s,i1)}::Parenthised (xxs,info_parens)::xs
+    when List.mem s dynamic_macs ||
+         not(Hashtbl.mem defs s) && List.exists (around info_parens) pos ->
+      (* drop out the argument list if it contains a parse error *)
+      let commas =
+	match info_parens with
+	  _::rest ->
+	    (match List.rev rest with
+	      _::commas -> commas
+	    | [] -> [])
+	| [] -> [] in
+      msg_apply_known_macro s;
+      [Parenthised (xxs, commas)]
+	+> iter_token_paren (set_as_comment Token_c.CppMacro);
+      let dynamic_macs =
+	if List.mem s dynamic_macs then dynamic_macs else s::dynamic_macs in
+      apply_macro_defs dynamic_macs pos xs
+
+  | ((PToken ({tok = TDefine i} as t1)) as start)::xs
+    when List.exists (at t1) pos ->
+      let (in_def,end_and_after) =
+	Common.span (function PToken {tok = TDefEOL _} -> false | _ -> true)
+	  xs in
+      (match end_and_after with
+	((PToken {tok = TDefEOL _}) as t2)::xs ->
+	  (* not reusable, like macros, so only comments out one #define
+	     per round *)
+	  [start] +> iter_token_paren (set_as_comment Token_c.CppMacro);
+	  in_def +> iter_token_paren (set_as_comment Token_c.CppMacro);
+	  [t2] +> iter_token_paren (set_as_comment Token_c.CppMacro);
+	  msg_apply_known_macro "#define";
+	  let pos =
+	    let mypos = (t1.Token_views_c.line,t1.Token_views_c.col) in
+	    List.filter (function x -> not (x = mypos)) pos in
+	  apply_macro_defs dynamic_macs pos xs
+      | _ -> failwith "#define with no end")
+
+  | PToken ({tok = TIdent (s,i1)} as id)::
+    PToken ({tok = TPtVirg _} | {tok = TEq _})::xs
+      when List.mem s !Flag.cocci_attribute_names ->
+	id.tok <- TMacroEndAttr (s, i1);
+	apply_macro_defs dynamic_macs pos xs
+
+  | PToken ({tok = TIdent (s,i1)} as id)::xs
+      when List.mem s !Flag.cocci_attribute_names ->
+	id.tok <- TMacroAttr (s, i1);
+	apply_macro_defs dynamic_macs pos xs
 
   | PToken ({tok = TIdent (s,i1)} as id)::xs
       when Hashtbl.mem defs s ->
@@ -469,18 +544,15 @@ let rec apply_macro_defs
                 id.tok <- token_from_parsinghack_hint (s,i1) hint;
           )
       );
-      apply_macro_defs xs
-
-
-
+      apply_macro_defs dynamic_macs pos xs
 
   (* recurse *)
-  | (PToken x)::xs -> apply_macro_defs xs
+  | (PToken x)::xs -> apply_macro_defs dynamic_macs pos xs
   | (Parenthised (xxs, info_parens))::xs ->
-      xxs +> List.iter apply_macro_defs;
-      apply_macro_defs xs
+      xxs +> List.iter (apply_macro_defs dynamic_macs pos);
+      apply_macro_defs dynamic_macs pos xs
  in
- apply_macro_defs xs
+ apply_macro_defs [] pos xs
 
 
 
