@@ -81,7 +81,7 @@ let new_test_parse_gen xs =
     pr2 ("PARSING: " ^ file);
 
     (* test parsing of format strings as well *)
-    let (xs, stat) = Parse_c.parse_c_and_cpp true file in
+    let (xs, stat) = Parse_c.parse_c_and_cpp true false file in
     xs +> List.iter (fun (ast, (s, toks)) ->
       Parse_c.print_commentized toks
     );
@@ -154,7 +154,7 @@ let test_parse_gen xs ext =
     pr2 ("PARSING: " ^ file);
 
     (* test parsing of format strings as well *)
-    let (xs, stat) = Parse_c.parse_c_and_cpp true file in
+    let (xs, stat) = Parse_c.parse_c_and_cpp true false file in
     xs +> List.iter (fun (ast, (s, toks)) ->
       Parse_c.print_commentized toks
     );
@@ -230,7 +230,7 @@ let local_test_cfg launchgv file =
   then pr2 "warning: seems not a .c file";
 
   (* no point to parse format strings *)
-  let (program, _stat) = Parse_c.parse_c_and_cpp false file in
+  let (program, _stat) = Parse_c.parse_c_and_cpp false false file in
 
   program +> List.iter (fun (e,_) ->
     let toprocess =
@@ -266,7 +266,7 @@ let local_test_cfg launchgv file =
 	      else
 		let fl = Filename.chop_extension (Filename.basename file) in
 		fl^":"^fn^".dot" in
-            Ograph_extended.print_ograph_mutable flow' (filename) launchgv
+            Control_flow_c.G.print_ograph_mutable flow' (filename) launchgv
           )
         with Ast_to_flow.Error (x) -> Ast_to_flow.report_error x
       )
@@ -278,7 +278,7 @@ let test_cfg = local_test_cfg true
 let test_cfg_ifdef file =
   Flag_parsing_c.ifdef_to_if := true;
   (* no point to parse format strings *)
-  let (ast2, _stat) = Parse_c.parse_c_and_cpp false file in
+  let (ast2, _stat) = Parse_c.parse_c_and_cpp false false file in
   let ast = Parse_c.program_of_program2 ast2 in
 
   ast +> List.iter (fun e ->
@@ -287,7 +287,7 @@ let test_cfg_ifdef file =
         flow +> do_option (fun flow ->
           Ast_to_flow.deadcode_detection flow;
           let flow = Ast_to_flow.annotate_loop_nodes flow in
-          Ograph_extended.print_ograph_mutable flow ("/tmp/output.dot") true
+          Control_flow_c.G.print_ograph_mutable flow ("/tmp/output.dot") true
         )
       with Ast_to_flow.Error (x) -> Ast_to_flow.report_error x
     )
@@ -299,7 +299,7 @@ let test_parse_unparse infile =
   then pr2 "warning: seems not a .c file";
 
   (* test parsing of format strings *)
-  let (program2, _stat) = Parse_c.parse_c_and_cpp true infile in
+  let (program2, _stat) = Parse_c.parse_c_and_cpp true false infile in
   let program2_with_ppmethod =
     program2 +> List.map (fun x -> x, Unparse_c.PPnormal)
   in
@@ -337,7 +337,7 @@ let test_type_c infile =
   Flag_parsing_c.pretty_print_type_info := true;
 
   (* no point to parse format strings *)
-  let (program2, _stat) =  Parse_c.parse_c_and_cpp false infile in
+  let (program2, _stat) =  Parse_c.parse_c_and_cpp false false infile in
   let _program2 =
     program2
     +> Common.unzip
@@ -360,7 +360,7 @@ let test_type_c infile =
 (* ex: demos/platform_ifdef.c *)
 let test_comment_annotater infile =
   (* no point to parse format strings *)
-  let (program2, _stat) =  Parse_c.parse_c_and_cpp false infile in
+  let (program2, _stat) =  Parse_c.parse_c_and_cpp false false infile in
   let asts = program2 +> List.map (fun (ast,_) -> ast) in
   let toks = program2 +> List.map (fun (ast, (s, toks)) -> toks) +>
     List.flatten in
@@ -413,7 +413,7 @@ let test_compare_c_hardcoded () =
 (* ---------------------------------------------------------------------- *)
 let test_attributes file =
   (* no point to parse format strings *)
-  let (ast2, _stat) = Parse_c.parse_c_and_cpp false file in
+  let (ast2, _stat) = Parse_c.parse_c_and_cpp false false file in
   let ast = Parse_c.program_of_program2 ast2 in
 
   Visitor_c.vk_program { Visitor_c.default_visitor_c with
@@ -427,12 +427,13 @@ let test_attributes file =
           xs +> List.iter (fun (onedecl, iicomma) ->
 
             let sattr  = Ast_c.s_of_attr onedecl.v_attr in
+            let sendattr  = Ast_c.s_of_attr onedecl.v_endattr in
             let idname =
               match onedecl.v_namei with
               | Some (name, ini) -> Ast_c.str_of_name name
               | None -> "novar"
             in
-            pr2 (spf "%-30s: %s" idname sattr);
+            pr2 (spf "%-30s: %s %s" idname sattr sendattr);
           );
       | _ -> ()
 
@@ -450,7 +451,7 @@ let cpp_options () = [
 let test_cpp file =
   Flag_parsing_c.ifdef_to_if := true;
   (* no point to parse format strings *)
-  let (ast2, _stat) = Parse_c.parse_c_and_cpp false file in
+  let (ast2, _stat) = Parse_c.parse_c_and_cpp false false file in
   let dirname = Filename.dirname file in
   let ast = Parse_c.program_of_program2 ast2 in
   let _ast = Cpp_ast_c.cpp_expand_include (cpp_options()) dirname ast in
@@ -542,7 +543,7 @@ let test_parse xs =
     pr2 ("PARSING: " ^ file);
 
     (* test parsing of format strings *)
-    let (xs, stat) = Parse_c.parse_c_and_cpp true file in
+    let (xs, stat) = Parse_c.parse_c_and_cpp true false file in
     xs +> List.iter (fun (ast, (s, toks)) ->
       Parse_c.print_commentized toks
     );
