@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -25,13 +25,20 @@ val statement_dots :
 val declaration_dots :
     Ast0_cocci.declaration Ast0_cocci.dots ->
       Ast_cocci.annotated_decl Ast_cocci.dots
+val field_dots :
+    Ast0_cocci.field Ast0_cocci.dots ->
+      Ast_cocci.annotated_field Ast_cocci.dots
+val define_param_dots :
+    Ast0_cocci.define_param Ast0_cocci.dots ->
+      Ast_cocci.define_param Ast_cocci.dots
 val case_line : Ast0_cocci.case_line -> Ast_cocci.case_line
 val string_fragment : Ast0_cocci.string_fragment -> Ast_cocci.string_fragment
 val typeC : bool (*allminus*) -> Ast0_cocci.typeC -> Ast_cocci.fullType
 val declaration : Ast0_cocci.declaration -> Ast_cocci.declaration
+val field : Ast0_cocci.field -> Ast_cocci.field
 val parameterTypeDef :
     Ast0_cocci.parameterTypeDef -> Ast_cocci.parameterTypeDef
 val parameter_list : Ast0_cocci.parameter_list -> Ast_cocci.parameter_list
 val top_level : Ast0_cocci.top_level -> Ast_cocci.top_level
-val mcode : 'a Ast0_cocci.mcode -> 'a Ast_cocci.mcode
 val convert_info : Ast0_cocci.info -> Ast_cocci.info
+val constraints : Ast0_cocci.constraints -> Ast_cocci.constraints
