@@ -262,7 +262,7 @@ let contains_added_strings info =
     List.exists
       (fun (str,_) ->
 	match str with
-	  (Ast.Noindent s | Ast.Indent s) -> not (String.trim s = "")
+	  (Ast.Noindent s | Ast.Indent s) -> not (Stdcompat.String.trim s = "")
 	| Ast.Space _ -> true (* adds a thing with space around it *))
       l in
   (* If this is true, eg if (x) return 0; will be broken up into
@@ -376,7 +376,8 @@ let classify is_minus all_marked table code =
   let ident r k e =
     compute_result Ast0.ident e
       (match Ast0.unwrap e with
-	Ast0.DisjId(starter,id_list,_,ender) ->
+	Ast0.DisjId(starter,id_list,_,ender)
+      | Ast0.ConjId(starter,id_list,_,ender) ->
 	  disj_cases e starter id_list r.VT0.combiner_rec_ident ender
       |	_ -> k e) in
 
@@ -575,7 +576,9 @@ let equal_ident i1 i2 =
   | (Ast0.MetaLocalFunc(name1,_,_),Ast0.MetaLocalFunc(name2,_,_)) ->
       equal_mcode name1 name2
   | (Ast0.DisjId(starter1,_,mids1,ender1),
-     Ast0.DisjId(starter2,_,mids2,ender2)) ->
+     Ast0.DisjId(starter2,_,mids2,ender2))
+  | (Ast0.ConjId(starter1,_,mids1,ender1),
+     Ast0.ConjId(starter2,_,mids2,ender2)) ->
       equal_mcode starter1 starter2 &&
       List.for_all2 equal_mcode mids1 mids2 &&
       equal_mcode ender1 ender2
@@ -623,10 +626,7 @@ let rec equal_expression e1 e2 =
       equal_mcode name1 name2
   | (Ast0.EComma(cm1),Ast0.EComma(cm2)) -> equal_mcode cm1 cm2
   | (Ast0.DisjExpr(starter1,_,mids1,ender1),
-     Ast0.DisjExpr(starter2,_,mids2,ender2)) ->
-       equal_mcode starter1 starter2 &&
-       List.for_all2 equal_mcode mids1 mids2 &&
-       equal_mcode ender1 ender2
+     Ast0.DisjExpr(starter2,_,mids2,ender2))
   | (Ast0.ConjExpr(starter1,_,mids1,ender1),
      Ast0.ConjExpr(starter2,_,mids2,ender2)) ->
        equal_mcode starter1 starter2 &&
@@ -944,6 +944,8 @@ let root_equal e1 e2 =
       dots equal_define_param d1 d2
   | (Ast0.IdentTag(i1),Ast0.IdentTag(i2)) -> equal_ident i1 i2
   | (Ast0.ExprTag(e1),Ast0.ExprTag(e2)) -> equal_expression e1 e2
+  | (Ast0.AssignOpTag(e1),Ast0.AssignOpTag(e2)) -> assignOp_equal e1 e2
+  | (Ast0.BinaryOpTag(e1),Ast0.BinaryOpTag(e2)) -> binaryOp_equal e1 e2
   | (Ast0.ArgExprTag(d),_) -> failwith "not possible - iso only"
   | (Ast0.TypeCTag(t1),Ast0.TypeCTag(t2)) -> equal_typeC t1 t2
   | (Ast0.ParamTag(p1),Ast0.ParamTag(p2)) -> equal_parameterTypeDef p1 p2
