@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -46,4 +46,4 @@ val print_minus_flag : bool ref
 val print_rule_elem : Ast_cocci.rule_elem -> unit
 val print_when_modif : Ast_cocci.when_modifier -> unit
 
-val dep : bool -> Ast_cocci.dependency -> unit
+val dependency : Ast_cocci.dependency -> unit
