@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -29,8 +29,7 @@ let contains_modif =
   let do_nothing r k e = k e in
   let annotated_decl decl =
     match Ast.unwrap decl with
-      Ast.DElem(bef,_,_) -> bef
-    | _ -> failwith "not possible" in
+      Ast.DElem(bef,_,_) -> bef in
   let rule_elem r k re =
     let res = k re in
     match Ast.unwrap re with
@@ -45,8 +44,9 @@ let contains_modif =
       mcode mcode mcode mcode mcode mcode mcode mcode mcode
       mcode mcode mcode mcode mcode
       do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-      do_nothing do_nothing do_nothing do_nothing
       do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
+      do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
+      do_nothing do_nothing
       do_nothing rule_elem do_nothing do_nothing do_nothing do_nothing in
   recursor.V.combiner_rule_elem
 
@@ -115,9 +115,9 @@ let ctl_au x y = CTL.AU(CTL.FORWARD,CTL.STRICT,x,y)
 
 let ctl_uncheck x  = CTL.Uncheck(x)
 
-let make_meta_rule_elem d =
+let make_meta_rule_elem d cstr =
   let nm = "_S" in
-  Ast.make_meta_rule_elem nm d ([],[],[])
+  Ast.make_meta_rule_elem nm d cstr ([],[],[])
 
 (* --------------------------------------------------------------------- *)
 
@@ -196,7 +196,7 @@ and ifthen keep_wit a test thn aft =
     | (_,Some a) ->
 	ctl_ax
 	  (ctl_and
-	     (predmaker keep_wit (make_meta_rule_elem aft))
+	     (predmaker keep_wit (make_meta_rule_elem aft Ast.CstrTrue))
 	     (ctl_ax a)) in
   let body =
     ctl_or
