@@ -61,6 +61,8 @@ let rec test_loop previous_merges cocci_file cfiles =
   match pending_instance with
     None -> (cocci_infos, res)
   | Some (cfiles', virt_rules, virt_ids) ->
+      (* don't support line ranges for iteration for now *)
+      let cfiles' = List.map (fun x -> (x,None)) cfiles' in
       Flag.defined_virtual_rules := virt_rules;
       Flag.defined_virtual_env := virt_ids;
       Common.clear_pr2_once ();
@@ -78,25 +80,32 @@ let test_with_output_redirected cocci_file cfiles expected_out =
   let current_out = end_redirect_output redirected_output in
   (res, current_out)
 
+let adjust_extension extension =
+  if extension = ""
+  then extension
+  else if String.get extension 0 = '.'
+  then String.sub extension 1 (String.length extension - 1)
+  else extension
+
 (* There can be multiple .c for the same cocci file. The convention
  * is to have one base.cocci and a base.c and some optional
  * base_vernn.[c,res].
  *
  * If want to test without iso, use -iso_file empty.iso option.
  *)
-let testone prefix x compare_with_expected_flag =
+let testone prefix x compare_with_expected =
   let x    = if x =~ "\\(.*\\)_ver0$" then matched1 x else x in
   let base = if x =~ "\\(.*\\)_ver[0-9]+$" then matched1 x else x in
 
-  let cfile      = prefix ^ x ^ ".c" in
+  let cfile =
+    if !Flag.c_plus_plus then prefix ^ x ^ ".cpp" else prefix ^ x ^ ".c" in
   let cocci_file = prefix ^ base ^ ".cocci" in
 
   let expected_out = prefix ^ base ^ out_suffix in
 
-  let expected_res   = prefix ^ x ^ ".res" in
   begin
     let (res, current_out) =
-      test_with_output_redirected cocci_file [cfile] expected_out in
+      test_with_output_redirected cocci_file [(cfile,None)] expected_out in
     let generated =
       match Common.optionise (fun () -> List.assoc cfile res) with
       | Some (Some outfile) ->
@@ -113,9 +122,11 @@ let testone prefix x compare_with_expected_flag =
           cfile
       | None -> raise (Impossible 163)
     in
-    if compare_with_expected_flag
-    then
-      begin
+    match compare_with_expected with
+      None -> ()
+    | Some extension ->
+	let expected_res =
+	  prefix ^ x ^ "." ^ (adjust_extension extension) in
 	Compare_c.compare_default generated expected_res
 	  +> Compare_c.compare_result_to_string
 	  +> pr2;
@@ -125,7 +136,6 @@ let testone prefix x compare_with_expected_flag =
 	    Compare_c.exact_compare current_out' expected_out
 	      +> Compare_c.compare_result_to_string
 	      +> pr2
-      end
   end;
   Common.erase_temp_files ()
 
@@ -206,7 +216,8 @@ let testall_bis extra_test expected_score_file update_score_file =
 	  pr2 res;
 
 	  let (xs, current_out) =
-	    test_with_output_redirected cocci_file [cfile] expected_out in
+	    test_with_output_redirected cocci_file [(cfile,None)]
+	      expected_out in
 
           let generated =
             match List.assoc cfile xs with
@@ -387,6 +398,7 @@ let test_okfailed cocci_file cfiles =
     try (
       Common.timeout_function_opt "testing" !Flag_cocci.timeout (fun () ->
 
+	let cfiles = List.map (fun x -> (x,None)) cfiles in (* no range *)
 	let (outfiles, current_out) =
 	  test_with_output_redirected cocci_file cfiles expected_out in
 
@@ -511,14 +523,15 @@ let test_regression_okfailed () =
 
 (* ------------------------------------------------------------------------ *)
 (* quite similar to test_ok_failed. Maybe could factorize code *)
-let compare_with_expected outfiles =
+let compare_with_expected outfiles extension =
+  let extension = adjust_extension extension in
   pr2 "";
   outfiles +> List.iter (fun (infile, outopt) ->
     let (dir, base, ext) = Common.dbe_of_filename infile in
     let expected_suffix   =
       match ext with
-      | "c" -> "res"
-      | "h" -> "h.res"
+      | "c" -> extension
+      | "h" -> "h."^extension
       | s -> failwith ("weird C file, not a .c or .h :" ^ s)
     in
     let expected_res =
